/*----------------------------------------------------------------------------* 
**  "comp3_1.cpp"  using numerical integration algorithm.
**
**  To build this in Visual C++ 6.0 perform the following steps:
**
**  1. Select New from File menu.
**
**  2. In the New dialog box select "Win32 Application" and enter 
**     a project name.  Then press the "Ok" button.
**
**  3. In the Win32 Application dialog box select "A simple Win32 Application" 
**     then press the "Finish" button.  
**
**  4. Press "Ok" in the New Project Information dialog box.
**
**  5. Open the *.cpp file for entry of the program.  (We do not edit StdAfx.h
**     or StdAfx.cpp; VC++ requires these files.)
*/

#include "stdafx.h"
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <windowsx.h>
#include "..\plotter\plotter.h"

plotter np( "Plotter", "Numerical Plotter" ); 

/*----------------------------------------------------------------------------*
**  "WinMain" is the program entry point
**
**  returns: message parameter
*/
int APIENTRY WinMain( HINSTANCE inst,           // window instance
                      HINSTANCE prev,           // previous window
                      LPSTR     cmdl,           // command line string
                      int       show )          // initial display
{
    np.initialize( inst );
    
    if( !np.window_class() )
	return 0;
  
    np.create( show );
 
    return np.message_loop();
}
    
/*----------------------------------------------------------------------------*
**  "WindowFunc" processes messages to this application
**
**  returns: 0 if message processed, else default value
*/
LRESULT CALLBACK WindowFunc( HWND   hwnd,       // handle to window of message
			     UINT   msg,        // message code
			     WPARAM wp,         // short message amplifier
			     LPARAM lp )        // long message amplifier
{
    switch( msg )
    {
    case WM_CREATE:
	np.on_create( hwnd );
	break;

    case WM_DESTROY:
	np.on_destroy();
	break;

    case WM_SIZE:
	np.on_size( hwnd );
	break;

    case WM_HSCROLL:
	np.on_hscroll( hwnd, wp );
	break;

    case WM_VSCROLL:
	np.on_vscroll( hwnd, wp );
	break;

    case WM_PAINT:
	np.on_paint( hwnd );
	break;

    case WM_COMMAND:
	np.on_command( hwnd, wp );
	break;

    default:
	return DefWindowProc( hwnd, msg, wp, lp );
    }

    return 0;
}

/*----------------------------------------------------------------------------*\
**              Put your mathematical functions to plot here.                 **
\*----------------------------------------------------------------------------*/

int calculate( POINT *, POINT *, int );
double integrate( double, double, int, double (f)(double) );
double Y( double );

/*----------------------------------------------------------------------------*
**  "run" performs calculations and plots results.  
**
**  This is a shell in which the function calls for a specific calculation
**  are entered.
**
**  returns: nothing
*/
void run()
{
    POINT curve1[110], curve2[110];
    int i;
            
    // first, define a plot region 
    // this must be done before calculating points to be plotted
    np.plot_region( -0.1, 2.0, 2.0, -2.0 );

    i = calculate( curve1, curve2, 110 );

    // plot the results
    np.draw_axes();
    np.red_curve( curve1, i );
    np.blue_curve( curve2, i );
}

/*----------------------------------------------------------------------------*
**  "calculate" is a shell for the calculation algorithms.
**
**  returns: number of items in POINT array
*/
int calculate( POINT *a0, POINT *a1, int points )
{
    int i;
    double amps  = 0.0; 
    double t     = 0.0;
    double delta = 0.02;

    a0[0] = np.plot_point( 0, 0 );
    a1[0] = np.plot_point( 0, 0 );

    for( i = 1; i < points; i++ )
    {
	// next iteration
	amps += integrate( t, delta, 2, Y );
	t    += delta;
	
	// append results to curves
	a0[i] = np.plot_point( t, amps );
	a1[i] = np.plot_point( t, Y( t ) );
    }
    return i;
}

/*----------------------------------------------------------------------------* 
**  "integrate" finds the definite integral of a specified function
**  using trapezoidal integration.
**
**  returns: nothing
*/
double integrate( double t_beg,                 // lower limit of integral
	          double delta,                 // period for integration
		  int    n,                     // number of segments > 0 
		  double (f)(double t) )        // integrand function
{
    double area;
    double t = t_beg;                           // initialize time
    double dt = delta / n;                      // initialize step size
    int    i;                                   // loop counter

    area = f( t_beg ) / 2.0;                    // initial value of integral 

    if( n > 1 )                                 // compute the n - 1 terms
    {                                           //   in the summation
	for( i = 1; i < n; i++ )
	{
	    t    += dt;                         // increment time
	    area += f( t );                     // add next term to sum
	}
    }

    area += f( t_beg + delta ) / 2.0;           // add last term
    area *= dt;                                 // scale by step size

    return area;
}

const double TWO_PI = 4 * acos( 0 );

/*----------------------------------------------------------------------------*
**  "Y" is the integrand function for this problem.
**
**  returns: value corresponding to argument
*/
double Y( double t )
{
    return ( ( 1 - exp( -t ) ) * cos( TWO_PI * t ) );
}

