/*----------------------------------------------------------------------------* 
**  "comp6_2.cpp"  using the Runge-Kutta numerical differentiation method
**  for solving matrix differential equations.
**
**  To build this in Visual C++ 6.0 perform the following steps:
**
**  1. Select New from File menu.
**
**  2. In the New dialog box select "Win32 Application" and enter 
**     a project name.  Then press the "Ok" button.
**
**  3. In the Win32 Application dialog box select "A simple Win32 Application" 
**     then press the "Finish" button.  
**
**  4. Press "Ok" in the New Project Information dialog box.
**
**  5. Open the *.cpp file for entry of the program.  (We do not edit StdAfx.h
**     or StdAfx.cpp; VC++ requires these files.)
*/

#include "stdafx.h"
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <windowsx.h>
#include "..\plotter\plotter.h"
#include "..\vector\vector.h"

plotter np( "Plotter", "Numerical Plotter" ); 

/*----------------------------------------------------------------------------*
**  "WinMain" is the program entry point
**
**  returns: message parameter
*/
int APIENTRY WinMain( HINSTANCE inst,           // window instance
                      HINSTANCE prev,           // previous window
                      LPSTR     cmdl,           // command line string
                      int       show )          // initial display
{
    np.initialize( inst );
    
    if( !np.window_class() )
	return 0;
  
    np.create( show );
 
    return np.message_loop();
}
    
/*----------------------------------------------------------------------------*
**  "WindowFunc" processes messages to this application
**
**  returns: 0 if message processed, else default value
*/
LRESULT CALLBACK WindowFunc( HWND   hwnd,       // handle to window of message
			     UINT   msg,        // message code
			     WPARAM wp,         // short message amplifier
			     LPARAM lp )        // long message amplifier
{
    switch( msg )
    {
    case WM_CREATE:
	np.on_create( hwnd );
	break;

    case WM_DESTROY:
	np.on_destroy();
	break;

    case WM_SIZE:
	np.on_size( hwnd );
	break;

    case WM_HSCROLL:
	np.on_hscroll( hwnd, wp );
	break;

    case WM_VSCROLL:
	np.on_vscroll( hwnd, wp );
	break;

    case WM_PAINT:
	np.on_paint( hwnd );
	break;

    case WM_COMMAND:
	np.on_command( hwnd, wp );
	break;

    default:
	return DefWindowProc( hwnd, msg, wp, lp );
    }

    return 0;
}

/*----------------------------------------------------------------------------*\
**              Put your mathematical functions to plot here.                 **
\*----------------------------------------------------------------------------*/

int calculate( POINT *, POINT *, POINT *, int );
vector mat_rk( double, vector &, double, int, vector (g)( double, vector & ) );
vector G( double t, vector & );

/*----------------------------------------------------------------------------*
**  "run" performs calculations and plots results.  
**
**  This is a shell in which the function calls for a specific calculation
**  are entered.
**
**  returns: nothing
*/
void run()
{
    POINT curve0[110], curve1[110], curve2[110];
    int i;        
    
    // first, define a plot region 
    // this must be done before calculating points to be plotted
    np.plot_region( -0.1, 10.0, 12, -3 );

    // do your calculations
    i = calculate( curve0, curve1, curve2, 110 );

    // plot the results
    np.draw_axes();   
    np.red_curve( curve0, i );
    np.green_curve( curve1, i );
    np.blue_curve( curve2, i );
}

/*----------------------------------------------------------------------------*
**  "calculate" is a shell for the calculation algorithms.
**
**  returns: number of items in POINT array
*/
int calculate( POINT *a0,                       // curve 0 
	       POINT *a1,                       // curve 1  
	       POINT *a2,                       // curve 2  
	       int points )                     // count 
{
    int i;
    vector y; 
    double t     = 0.0;
    double delta = 0.1;
  
    a0[0] = np.plot_point( 0, 0 );
    a1[0] = np.plot_point( 0, 0 );
    a2[0] = np.plot_point( 0, 0 );

    for( i = 1; i < points; i++ )
    {
	// next iteration
	y  = mat_rk( t, y, delta, 2, G );
	t += delta;
	
	// append results to curves
	a0[i] = np.plot_point( t, y[0] );
	a1[i] = np.plot_point( t, y[1] );
	a2[i] = np.plot_point( t, y[2] );
    }
    return i;
}

/*----------------------------------------------------------------------------* 
**  "mat_rk" performs numerical differentiation using the 4-th order 
**  Runge Kutta algorithm to solve matrix differential equations of the form: 
**
**                      y' = g( t, y )
**
**  returns: nothing
*/
vector mat_rk( double  t_beg,                   // interval start
               vector &y_beg,                   // y at start
               double  delta,                   // interval duration
               int     n,                       // number of steps > 0
               vector (g)( double, vector & ) ) // integrand function
{
    vector y = y_beg;                           // initialize vector 
    double t = t_beg;                           // initialize time
    double dt = delta / n;                      // initialize step duration
    
    vector g1, g2, g3, g4;                      // variables for computation
    double dt2 = dt / 2.0;
    double dt6 = dt / 6.0;
    int    i;                                   // loop counter

    for( i = 0; i < n; i++ )                    // do n steps of R-K algorithm
    {
	g1 = g( t,       y );
	g2 = g( t + dt2, y + g1 * dt2 );
	g3 = g( t + dt2, y + g2 * dt2 );
	g4 = g( t + dt,  y + g3 * dt );

	y = y + ( g1 + g2 * 2.0 + g3 * 2.0 + g4 ) * dt6;
	t = t + dt;
    }

    return y;
}

/*----------------------------------------------------------------------------*
**  "G" defines a matrix differential equation of the form:
**
**            y' = G(t) y + u(t)
**
**  returns: the left hand side of the above
*/
vector G( double t, vector &y )
{
    vector g, u;

    // the input vector u(t)
    u[0] =  0;
    u[1] =  0;
    u[2] = 10;
    
    // the equation
    g[0] =   0.0 * y[0] + 1.0 * y[1] + 0.0 * y[2]  +  u[0];
    g[1] =   0.0 * y[0] + 0.0 * y[1] + 1.0 * y[2]  +  u[1];
    g[2] = - 1.0 * y[0] - 2.0 * y[1] - 2.0 * y[2]  +  u[2];

    return g;
}



